@@ -1,20 +1,5 @@
-/*
- * This file is part of Tepl, a text editor library.
- *
- * Copyright 2017 - Sébastien Wilmet <swilmet@gnome.org>
- *
- * Tepl is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by the
- * Free Software Foundation; either version 2.1 of the License, or (at your
- * option) any later version.
- *
- * Tepl is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
- * License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, see <http://www.gnu.org/licenses/>.
+/* SPDX-FileCopyrightText: 2017 - Sébastien Wilmet <swilmet@gnome.org>
+ * SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #include "config.h"
@@ -168,9 +153,7 @@ set_tab (TeplTabLabel *tab_label,
 	g_return_if_fail (TEPL_IS_TAB (tab));
 
 	g_assert (tab_label->priv->tab == NULL);
-	tab_label->priv->tab = tab;
-	g_object_add_weak_pointer (G_OBJECT (tab_label->priv->tab),
-				   (gpointer *) &tab_label->priv->tab);
+	g_set_weak_pointer (&tab_label->priv->tab, tab);
 
 	view = tepl_tab_get_view (tab);
 	g_signal_connect_object (view,
@@ -227,13 +210,7 @@ tepl_tab_label_dispose (GObject *object)
 {
 	TeplTabLabel *tab_label = TEPL_TAB_LABEL (object);
 
-	if (tab_label->priv->tab != NULL)
-	{
-		g_object_remove_weak_pointer (G_OBJECT (tab_label->priv->tab),
-					      (gpointer *) &tab_label->priv->tab);
-		tab_label->priv->tab = NULL;
-	}
-
+	g_clear_weak_pointer (&tab_label->priv->tab);
 	_tepl_signal_group_clear (&tab_label->priv->buffer_signal_group);
 	_tepl_signal_group_clear (&tab_label->priv->file_signal_group);
 
