@@ -1,32 +1,13 @@
-/*
- * This file is part of Tepl, a text editor library.
- *
- * Copyright 2016-2020 - Sébastien Wilmet <swilmet@gnome.org>
- *
- * Tepl is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by the
- * Free Software Foundation; either version 2.1 of the License, or (at your
- * option) any later version.
- *
- * Tepl is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
- * License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, see <http://www.gnu.org/licenses/>.
+/* SPDX-FileCopyrightText: 2016-2020 - Sébastien Wilmet <swilmet@gnome.org>
+ * SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #include "config.h"
 #include "tepl-tab.h"
 #include <glib/gi18n-lib.h>
 #include "tepl-close-confirm-dialog-single.h"
-#include "tepl-file-loader.h"
-#include "tepl-file-saver.h"
 #include "tepl-info-bar.h"
 #include "tepl-tab-group.h"
-#include "tepl-tab-saving.h"
-#include "tepl-utils.h"
 
 /**
  * SECTION:tab
@@ -65,7 +46,9 @@
 
 struct _TeplTabPrivate
 {
+	GtkScrolledWindow *scrolled_window;
 	TeplView *view;
+	TeplGotoLineBar *goto_line_bar;
 };
 
 enum
@@ -121,40 +104,33 @@ static void
 tepl_tab_pack_view_default (TeplTab  *tab,
 			    TeplView *view)
 {
-	GtkScrolledWindow *scrolled_window;
+	if (tab->priv->scrolled_window != NULL)
+	{
+		g_warning ("The TeplTab::pack_view virtual function can be called only once.");
+		return;
+	}
 
-	scrolled_window = create_scrolled_window ();
+	tab->priv->scrolled_window = create_scrolled_window ();
+	g_object_ref_sink (tab->priv->scrolled_window);
 
-	gtk_container_add (GTK_CONTAINER (scrolled_window),
+	gtk_container_add (GTK_CONTAINER (tab->priv->scrolled_window),
 			   GTK_WIDGET (view));
 
 	gtk_container_add (GTK_CONTAINER (tab),
-			   GTK_WIDGET (scrolled_window));
+			   GTK_WIDGET (tab->priv->scrolled_window));
 }
 
 static void
 tepl_tab_pack_info_bar_default (TeplTab    *tab,
 				GtkInfoBar *info_bar)
 {
-	GList *children;
-	GList *l;
 	GtkWidget *sibling = NULL;
 
-	children = gtk_container_get_children (GTK_CONTAINER (tab));
-
-	for (l = children; l != NULL; l = l->next)
+	if (tab->priv->scrolled_window != NULL)
 	{
-		GtkWidget *child = l->data;
-
-		if (!GTK_IS_INFO_BAR (child))
-		{
-			sibling = child;
-			break;
-		}
+		sibling = GTK_WIDGET (tab->priv->scrolled_window);
 	}
 
-	g_list_free (children);
-
 	if (sibling != NULL)
 	{
 		gtk_grid_insert_next_to (GTK_GRID (tab), sibling, GTK_POS_TOP);
@@ -171,6 +147,13 @@ tepl_tab_pack_info_bar_default (TeplTab    *tab,
 	}
 }
 
+static void
+tepl_tab_pack_goto_line_bar_default (TeplTab         *tab,
+				     TeplGotoLineBar *goto_line_bar)
+{
+	gtk_container_add (GTK_CONTAINER (tab), GTK_WIDGET (goto_line_bar));
+}
+
 static void
 close_confirm_dialog_single_cb (GObject      *source_object,
 				GAsyncResult *result,
@@ -288,7 +271,9 @@ tepl_tab_dispose (GObject *object)
 {
 	TeplTab *tab = TEPL_TAB (object);
 
+	g_clear_object (&tab->priv->scrolled_window);
 	g_clear_object (&tab->priv->view);
+	g_clear_object (&tab->priv->goto_line_bar);
 
 	G_OBJECT_CLASS (tepl_tab_parent_class)->dispose (object);
 }
@@ -304,6 +289,7 @@ tepl_tab_class_init (TeplTabClass *klass)
 
 	klass->pack_view = tepl_tab_pack_view_default;
 	klass->pack_info_bar = tepl_tab_pack_info_bar_default;
+	klass->pack_goto_line_bar = tepl_tab_pack_goto_line_bar_default;
 	klass->close_request = tepl_tab_close_request_default;
 
 	/**
@@ -456,6 +442,39 @@ tepl_tab_get_buffer (TeplTab *tab)
 	return TEPL_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (tab->priv->view)));
 }
 
+/**
+ * tepl_tab_get_goto_line_bar:
+ * @tab: a #TeplTab.
+ *
+ * Gets the #TeplGotoLineBar widget belonging to @tab. The #TeplGotoLineBar must
+ * not be destroyed by the application, the purpose of this function is to
+ * show/hide the widget.
+ *
+ * Returns: (transfer none): the #TeplGotoLineBar widget belonging to @tab.
+ * Since: 5.0
+ */
+TeplGotoLineBar *
+tepl_tab_get_goto_line_bar (TeplTab *tab)
+{
+	g_return_val_if_fail (TEPL_IS_TAB (tab), NULL);
+
+	if (tab->priv->goto_line_bar == NULL)
+	{
+		tab->priv->goto_line_bar = tepl_goto_line_bar_new ();
+		g_object_ref_sink (tab->priv->goto_line_bar);
+
+		/* The TeplGotoLineBar needs to be explicitly shown/hidden. */
+		gtk_widget_set_no_show_all (GTK_WIDGET (tab->priv->goto_line_bar), TRUE);
+
+		tepl_goto_line_bar_set_view (tab->priv->goto_line_bar,
+					     tab->priv->view);
+
+		TEPL_TAB_GET_CLASS (tab)->pack_goto_line_bar (tab, tab->priv->goto_line_bar);
+	}
+
+	return tab->priv->goto_line_bar;
+}
+
 /**
  * tepl_tab_add_info_bar:
  * @tab: a #TeplTab.
@@ -478,323 +497,3 @@ tepl_tab_add_info_bar (TeplTab    *tab,
 
 	TEPL_TAB_GET_CLASS (tab)->pack_info_bar (tab, info_bar);
 }
-
-static void
-load_file_content_cb (GObject      *source_object,
-		      GAsyncResult *result,
-		      gpointer      user_data)
-{
-	TeplFileLoader *loader = TEPL_FILE_LOADER (source_object);
-	TeplTab *tab = TEPL_TAB (user_data);
-	GError *error = NULL;
-
-	if (tepl_file_loader_load_finish (loader, result, &error))
-	{
-		TeplBuffer *buffer;
-		TeplFile *file;
-
-		buffer = tepl_tab_get_buffer (tab);
-		file = tepl_buffer_get_file (buffer);
-		tepl_file_add_uri_to_recent_manager (file);
-	}
-
-	if (error != NULL)
-	{
-		TeplInfoBar *info_bar;
-
-		info_bar = tepl_info_bar_new_simple (GTK_MESSAGE_ERROR,
-						     _("Error when loading the file."),
-						     error->message);
-
-		tepl_tab_add_info_bar (tab, GTK_INFO_BAR (info_bar));
-		gtk_widget_show (GTK_WIDGET (info_bar));
-
-		g_clear_error (&error);
-	}
-
-	g_object_unref (loader);
-	g_object_unref (tab);
-}
-
-/**
- * tepl_tab_load_file:
- * @tab: a #TeplTab.
- * @location: a #GFile.
- *
- * Unconditionally loads a file in @tab, regardless if there are unsaved changes
- * in the #GtkTextBuffer. The previous buffer content is lost.
- *
- * This function is asynchronous, there is no way to know when the file loading
- * is finished.
- *
- * Since: 4.0
- */
-void
-tepl_tab_load_file (TeplTab *tab,
-		    GFile   *location)
-{
-	TeplBuffer *buffer;
-	TeplFile *file;
-	TeplFileLoader *loader;
-
-	g_return_if_fail (TEPL_IS_TAB (tab));
-	g_return_if_fail (G_IS_FILE (location));
-
-	buffer = tepl_tab_get_buffer (tab);
-	file = tepl_buffer_get_file (buffer);
-
-	tepl_file_set_location (file, location);
-	loader = tepl_file_loader_new (buffer, file);
-
-	tepl_file_loader_load_async (loader,
-				     G_PRIORITY_DEFAULT,
-				     NULL, /* cancellable */
-				     NULL, NULL, NULL, /* progress */
-				     load_file_content_cb,
-				     g_object_ref (tab));
-}
-
-/**
- * tepl_tab_save_async:
- * @tab: a #TeplTab.
- * @callback: (scope async): a #GAsyncReadyCallback to call when the request is
- *   satisfied.
- * @user_data: user data to pass to @callback.
- *
- * Saves asynchronously the content of the @tab. The #TeplFile:location must not
- * be %NULL.
- *
- * See the #GAsyncResult documentation to know how to use this function.
- *
- * Since: 4.0
- */
-void
-tepl_tab_save_async (TeplTab             *tab,
-		     GAsyncReadyCallback  callback,
-		     gpointer             user_data)
-{
-	TeplBuffer *buffer;
-	TeplFile *file;
-	GFile *location;
-	TeplFileSaver *saver;
-
-	g_return_if_fail (TEPL_IS_TAB (tab));
-
-	buffer = tepl_tab_get_buffer (tab);
-	file = tepl_buffer_get_file (buffer);
-	location = tepl_file_get_location (file);
-	g_return_if_fail (location != NULL);
-
-	saver = tepl_file_saver_new (buffer, file);
-	_tepl_tab_saving_save_async (tab, saver, callback, user_data);
-	g_object_unref (saver);
-}
-
-/**
- * tepl_tab_save_finish:
- * @tab: a #TeplTab.
- * @result: a #GAsyncResult.
- *
- * Finishes a tab saving started with tepl_tab_save_async().
- *
- * Returns: whether the tab was saved successfully.
- * Since: 4.0
- */
-gboolean
-tepl_tab_save_finish (TeplTab      *tab,
-		      GAsyncResult *result)
-{
-	return _tepl_tab_saving_save_finish (tab, result);
-}
-
-static void
-save_async_simple_cb (GObject      *source_object,
-		      GAsyncResult *result,
-		      gpointer      user_data)
-{
-	TeplTab *tab = TEPL_TAB (source_object);
-
-	tepl_tab_save_finish (tab, result);
-	g_object_unref (tab);
-}
-
-/**
- * tepl_tab_save_async_simple:
- * @tab: a #TeplTab.
- *
- * The same as tepl_tab_save_async(), but without callback.
- *
- * This function is useful when you don't need to know:
- * - when the operation is finished;
- * - and whether the operation ran successfully.
- *
- * Since: 4.0
- */
-void
-tepl_tab_save_async_simple (TeplTab *tab)
-{
-	g_return_if_fail (TEPL_IS_TAB (tab));
-
-	g_object_ref (tab);
-	tepl_tab_save_async (tab,
-			     save_async_simple_cb,
-			     NULL);
-}
-
-static void
-save_as_cb (GObject      *source_object,
-	    GAsyncResult *result,
-	    gpointer      user_data)
-{
-	TeplTab *tab = TEPL_TAB (source_object);
-	GTask *task = G_TASK (user_data);
-	gboolean ok;
-
-	ok = _tepl_tab_saving_save_finish (tab, result);
-
-	g_task_return_boolean (task, ok);
-	g_object_unref (task);
-}
-
-static void
-save_file_chooser_response_cb (GtkFileChooserDialog *file_chooser_dialog,
-			       gint                  response_id,
-			       GTask                *task)
-{
-	if (response_id == GTK_RESPONSE_ACCEPT)
-	{
-		TeplTab *tab;
-		TeplBuffer *buffer;
-		TeplFile *file;
-		GFile *location;
-		TeplFileSaver *saver;
-
-		tab = g_task_get_source_object (task);
-		buffer = tepl_tab_get_buffer (tab);
-		file = tepl_buffer_get_file (buffer);
-
-		location = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (file_chooser_dialog));
-
-		saver = tepl_file_saver_new_with_target (buffer, file, location);
-		g_object_unref (location);
-
-		_tepl_tab_saving_save_async (tab, saver, save_as_cb, task);
-		g_object_unref (saver);
-	}
-	else
-	{
-		g_task_return_boolean (task, FALSE);
-		g_object_unref (task);
-	}
-
-	gtk_widget_destroy (GTK_WIDGET (file_chooser_dialog));
-}
-
-/**
- * tepl_tab_save_as_async:
- * @tab: a #TeplTab.
- * @callback: (scope async): a #GAsyncReadyCallback to call when the request is
- *   satisfied.
- * @user_data: user data to pass to @callback.
- *
- * Shows a #GtkFileChooser to save the @tab to a different location, creates an
- * appropriate #TeplFileSaver and asynchronously runs it.
- *
- * See the #GAsyncResult documentation to know how to use this function.
- *
- * Since: 4.0
- */
-void
-tepl_tab_save_as_async (TeplTab             *tab,
-			GAsyncReadyCallback  callback,
-			gpointer             user_data)
-{
-	GTask *task;
-	GtkWidget *file_chooser_dialog;
-	GtkFileChooser *file_chooser;
-
-	g_return_if_fail (TEPL_IS_TAB (tab));
-
-	task = g_task_new (tab, NULL, callback, user_data);
-
-	file_chooser_dialog = gtk_file_chooser_dialog_new (_("Save File"),
-							   NULL,
-							   GTK_FILE_CHOOSER_ACTION_SAVE,
-							   _("_Cancel"), GTK_RESPONSE_CANCEL,
-							   _("_Save"), GTK_RESPONSE_ACCEPT,
-							   NULL);
-
-	gtk_dialog_set_default_response (GTK_DIALOG (file_chooser_dialog), GTK_RESPONSE_ACCEPT);
-
-	/* Prevent tab from being destroyed. */
-	gtk_window_set_modal (GTK_WINDOW (file_chooser_dialog), TRUE);
-
-	_tepl_utils_associate_secondary_window (GTK_WINDOW (file_chooser_dialog),
-						GTK_WIDGET (tab));
-
-	file_chooser = GTK_FILE_CHOOSER (file_chooser_dialog);
-
-	gtk_file_chooser_set_do_overwrite_confirmation (file_chooser, TRUE);
-	gtk_file_chooser_set_local_only (file_chooser, FALSE);
-
-	g_signal_connect (file_chooser_dialog,
-			  "response",
-			  G_CALLBACK (save_file_chooser_response_cb),
-			  task);
-
-	gtk_widget_show (file_chooser_dialog);
-}
-
-/**
- * tepl_tab_save_as_finish:
- * @tab: a #TeplTab.
- * @result: a #GAsyncResult.
- *
- * Finishes a tab saving started with tepl_tab_save_as_async().
- *
- * Returns: whether the tab was saved successfully.
- * Since: 4.0
- */
-gboolean
-tepl_tab_save_as_finish (TeplTab      *tab,
-			 GAsyncResult *result)
-{
-	g_return_val_if_fail (TEPL_IS_TAB (tab), FALSE);
-	g_return_val_if_fail (g_task_is_valid (result, tab), FALSE);
-
-	return g_task_propagate_boolean (G_TASK (result), NULL);
-}
-
-static void
-save_as_async_simple_cb (GObject      *source_object,
-			 GAsyncResult *result,
-			 gpointer      user_data)
-{
-	TeplTab *tab = TEPL_TAB (source_object);
-
-	tepl_tab_save_as_finish (tab, result);
-	g_object_unref (tab);
-}
-
-/**
- * tepl_tab_save_as_async_simple:
- * @tab: a #TeplTab.
- *
- * The same as tepl_tab_save_as_async(), but without callback.
- *
- * This function is useful when you don't need to know:
- * - when the operation is finished;
- * - and whether the operation ran successfully.
- *
- * Since: 4.0
- */
-void
-tepl_tab_save_as_async_simple (TeplTab *tab)
-{
-	g_return_if_fail (TEPL_IS_TAB (tab));
-
-	g_object_ref (tab);
-	tepl_tab_save_as_async (tab,
-				save_as_async_simple_cb,
-				NULL);
-}
