@@ -1,20 +1,5 @@
-/*
- * This file is part of Tepl, a text editor library.
- *
- * Copyright 2016 - Sébastien Wilmet <swilmet@gnome.org>
- *
- * Tepl is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by the
- * Free Software Foundation; either version 2.1 of the License, or (at your
- * option) any later version.
- *
- * Tepl is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
- * License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, see <http://www.gnu.org/licenses/>.
+/* SPDX-FileCopyrightText: 2016 - Sébastien Wilmet <swilmet@gnome.org>
+ * SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #include <tepl/tepl.h>
@@ -224,6 +209,7 @@ main (gint    argc,
 {
 	GtkWidget *window;
 
+	tepl_init ();
 	gtk_init (&argc, &argv);
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -237,6 +223,6 @@ main (gint    argc,
 			  NULL);
 
 	gtk_main ();
-
+	tepl_finalize ();
 	return EXIT_SUCCESS;
 }
