@@ -1,24 +1,140 @@
-/*
- * This file is part of Tepl, a text editor library.
- *
- * Copyright 2016 - Sébastien Wilmet <swilmet@gnome.org>
- *
- * Tepl is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by the
- * Free Software Foundation; either version 2.1 of the License, or (at your
- * option) any later version.
- *
- * Tepl is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
- * License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, see <http://www.gnu.org/licenses/>.
+/* SPDX-FileCopyrightText: 2016-2020 - Sébastien Wilmet <swilmet@gnome.org>
+ * SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
+#include "config.h"
 #include <tepl/tepl.h>
-#include <unistd.h>
+#include <glib/gi18n-lib.h>
+#include "tepl-test-utils.h"
+
+static void
+check_short_name (TeplFile    *file,
+		  const gchar *expected_short_name)
+{
+	gchar *received_short_name;
+
+	received_short_name = tepl_file_get_short_name (file);
+	g_assert_cmpstr (received_short_name, ==, expected_short_name);
+	g_free (received_short_name);
+}
+
+static void
+check_short_name_is_untitled_file_number (TeplFile *file,
+					  gint      untitled_number)
+{
+	gchar *expected_short_name;
+
+	/* For the translation it needs to be the exact same string as in the
+	 * TeplFile implementation, to be able to run the unit test with any
+	 * locale.
+	 */
+	expected_short_name = g_strdup_printf (_("Untitled File %d"), untitled_number);
+	check_short_name (file, expected_short_name);
+	g_free (expected_short_name);
+}
+
+static void
+test_untitled_files (void)
+{
+	TeplFile *file1;
+	TeplFile *file2;
+	TeplFile *file3;
+	GFile *location;
+
+	file1 = tepl_file_new ();
+	check_short_name_is_untitled_file_number (file1, 1);
+
+	file2 = tepl_file_new ();
+	check_short_name_is_untitled_file_number (file2, 2);
+
+	/* Release an untitled number by destroying a file. */
+	g_object_unref (file1);
+	check_short_name_is_untitled_file_number (file2, 2); // still the same.
+	file1 = tepl_file_new ();
+	check_short_name_is_untitled_file_number (file1, 1);
+
+	/* Release an untitled number by setting a location. */
+	location = g_file_new_for_path ("location");
+	tepl_file_set_location (file1, location);
+	check_short_name_is_untitled_file_number (file2, 2); // still the same.
+	file3 = tepl_file_new ();
+	check_short_name_is_untitled_file_number (file3, 1);
+
+	/* Reset location to NULL. */
+	tepl_file_set_location (file1, NULL);
+	check_short_name_is_untitled_file_number (file1, 3);
+
+	g_object_unref (file1);
+	g_object_unref (file2);
+	g_object_unref (file3);
+	g_object_unref (location);
+}
+
+static void
+test_short_name (void)
+{
+	TeplFile *file;
+	GFile *location;
+	TeplWaitSignalData *data;
+	GError *error = NULL;
+
+	location = g_file_new_build_filename (g_get_tmp_dir (), "tepl-test-file", NULL);
+
+	/* Get the fallback short-name, for a file that doesn't exist. */
+	g_file_delete (location, NULL, &error);
+	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
+	{
+		g_clear_error (&error);
+	}
+	g_assert_no_error (error);
+
+	file = tepl_file_new ();
+	data = _tepl_test_utils_wait_signal_setup (G_OBJECT (file), "notify::short-name");
+	tepl_file_set_location (file, location);
+	_tepl_test_utils_wait_signal (data);
+	check_short_name (file, "tepl-test-file");
+	g_object_unref (file);
+
+	/* Get the display-name for a local file (so the file must exist). */
+	_tepl_test_utils_set_file_content (location, "file content");
+
+	file = tepl_file_new ();
+	data = _tepl_test_utils_wait_signal_setup (G_OBJECT (file), "notify::short-name");
+	tepl_file_set_location (file, location);
+	_tepl_test_utils_wait_signal (data);
+	check_short_name (file, "tepl-test-file");
+	g_object_unref (file);
+	g_object_unref (location);
+
+	/* Test the special case for a remote location that has no parent GFile. */
+	location = g_file_new_for_uri ("https://swilmet.be");
+	file = tepl_file_new ();
+	data = _tepl_test_utils_wait_signal_setup (G_OBJECT (file), "notify::short-name");
+	tepl_file_set_location (file, location);
+	_tepl_test_utils_wait_signal (data);
+	check_short_name (file, "https://swilmet.be");
+	g_object_unref (file);
+	g_object_unref (location);
+
+	// It's not really important if the trailing slash is still present or
+	// not in the short-name, but test it anyway. The important thing is to
+	// have the https://swilmet.be prefix, with an optional trailing slash.
+	location = g_file_new_for_uri ("https://swilmet.be/");
+	file = tepl_file_new ();
+	data = _tepl_test_utils_wait_signal_setup (G_OBJECT (file), "notify::short-name");
+	tepl_file_set_location (file, location);
+	_tepl_test_utils_wait_signal (data);
+	check_short_name (file, "https://swilmet.be/");
+	g_object_unref (file);
+	g_object_unref (location);
+}
+
+#if 0
+static void
+sleep_for_one_second (void)
+{
+	g_usleep (G_USEC_PER_SEC);
+}
 
 static void
 load_cb (GObject      *source_object,
@@ -183,7 +299,7 @@ test_externally_modified (void)
 	/* Modify externally and then save.
 	 * Sleep one second to force the timestamp/etag to change.
 	 */
-	sleep (1);
+	sleep_for_one_second ();
 	g_file_set_contents (path, "b", -1, &error);
 	g_assert_no_error (error);
 	tepl_file_check_file_on_disk (file);
@@ -195,7 +311,7 @@ test_externally_modified (void)
 	g_assert_true (!tepl_file_is_externally_modified (file));
 
 	/* Modify externally and then load */
-	sleep (1);
+	sleep_for_one_second ();
 	g_file_set_contents (path, "c", -1, &error);
 	g_assert_no_error (error);
 	tepl_file_check_file_on_disk (file);
@@ -207,7 +323,7 @@ test_externally_modified (void)
 	g_assert_true (!tepl_file_is_externally_modified (file));
 
 	/* Modify externally and then save as */
-	sleep (1);
+	sleep_for_one_second ();
 	g_file_set_contents (path, "d", -1, &error);
 	g_assert_no_error (error);
 	tepl_file_check_file_on_disk (file);
@@ -225,7 +341,7 @@ test_externally_modified (void)
 	g_assert_true (!tepl_file_is_externally_modified (file));
 
 	/* Modify externally and then save as to same location */
-	sleep (1);
+	sleep_for_one_second ();
 	g_file_set_contents (new_path, "f", -1, &error);
 	g_assert_no_error (error);
 	tepl_file_check_file_on_disk (file);
@@ -249,14 +365,20 @@ test_externally_modified (void)
 	g_object_unref (new_location);
 	g_object_unref (buffer);
 }
+#endif
 
-gint
-main (gint   argc,
-      gchar *argv[])
+int
+main (int    argc,
+      char **argv)
 {
 	gtk_test_init (&argc, &argv);
 
-	g_test_add_func ("/file/externally-modified", test_externally_modified);
+	g_test_add_func ("/file/untitled_files", test_untitled_files);
+	g_test_add_func ("/file/short_name", test_short_name);
+
+#if 0
+	g_test_add_func ("/file/externally_modified", test_externally_modified);
+#endif
 
 	return g_test_run ();
 }
